@@ -4,6 +4,7 @@ use strict;
 use warnings;
 
 our $VERSION=0.1;
+our $WARN=0;
 
 =head1 NAME
 
@@ -16,7 +17,16 @@ Net::vCard - Read and write vCard files (RFC 2426). vCard files hold personal in
   my $cards=Net::vCard->loadFile( "addresses.vcf" );
 
   foreach my $card ( @$cards ) {
-    print $card->{'N'}{'firstName'}, " ", $card->{'N'}{'lastName'}, "\n";
+
+    print $card->givenName,       " ",  $card->familyName, "\n";
+    print $card->ADR->address,    "\n"; 
+    print $card->ADR->city,       " ",  $card->ADR->region, "\n";
+    print $card->ADR->postalCode, "\n";
+
+    print $card->ADR("home")->address,    "\n"; 
+    print $card->ADR("home")->city,       " ",  $card->ADR("home")->region, "\n";
+    print $card->ADR("home")->postalCode, "\n";
+
   }
 
 =head1 MODULE STATUS
@@ -37,6 +47,8 @@ For those who really want to use this module right away
 
 
 use base qw(Net::vFile);
+use Net::vCard::ADR;
+
 $Net::vFile::classMap{'VCARD'}=__PACKAGE__;
 
 =head1 ACCESSOR METHODS
@@ -104,14 +116,79 @@ sub prefixes {
 
 =head2 ADDRESSES
 
+To access address data:
+
+ $card->ADR( type )->field;
+ $card->ADR( )->city;           # Default address, city field
+ $card->ADR( "home" )->address; # Home address type, address field
+
 =over 4
 
-=cut
+=item $card->ADR( [type] )->country
+
+=item $card->ADR( [type] )->poBox
+
+=item $card->ADR( [type] )->city
+
+=item $card->ADR( [type] )->region
+
+=item $card->ADR( [type] )->address
+
+=item $card->ADR( [type] )->postalCode
+
+=item $card->ADR( [type] )->extendedAddress
 
 =back
 
+
+There are some decisions to be taken wrt ADR values. 
+
+Firstly
+
+As of now the RFC specifies
+action to take in the case of unlisted type - the address gets four types - intl,
+parcel, postal, and work. This implies that several types refer to the same address.
+
+What I am doing for loading this data is storing the address in a hash entry by
+the first name and listing the remainder in "_alias" hash key.
+
+What happens when one of these addresses is updated? Do we copy all the values to
+unique hash entries or do we update the common copy, requiring the developer to
+explicitly declare a new address replace the common entry.
+
+If this doesn't make sense email me and I'll try another explaination.
+
+Secondly
+
+What about preferred addresses? For now I am going to let the module user optionally
+request their preferred address type. If it does not exist then we'll keep looking
+for less preferred address types like the "pref" that was specified when loading vcard
+data, and finally the 4 default types.
+
 =cut
 
+sub ADR  {
+
+    my $self=shift;
+    my $reqType=shift || $self->{'ADR'}{'_pref'};
+
+    foreach my $type ( $reqType, @{$self->typeDefault->{'ADR'}} ) {
+        next unless $type;
+        if (exists $self->{'ADR'}{$type}) {
+            return $self->{'ADR'}{$type};
+        }
+
+        if (exists $self->{'ADR'}{'_alias'}{$type}) {
+            return $self->{'ADR'}{'_alias'}{$type};
+        }
+    }
+
+    warn "No address found\n" if $WARN;
+    my $adrPkg=ref($self) . "::ADR";
+    return $adrPkg->new;
+
+}
+
 sub FN   { $_[0]->_singleText( "FN", $_[1] ); }
 sub BDAY { $_[0]->_singleText( "BDAY", $_[1] ); }
 
@@ -210,15 +287,17 @@ sub load_ADR {
 	# What to do about comma separated things?
 
     my $actual=shift @types;
-	$_[0]->{$_[1]}{$actual} = {
-		poBox      => $parts[0],
-		extended_address => $parts[1],
-		address    => $parts[2],
-		city       => $parts[3],
-		region     => $parts[4],
-		postalCode => $parts[5],
-		country    => $parts[6],
-	};
+    my $adrPkg = ref($_[0]) . "::ADR";
+
+	$_[0]->{$_[1]}{$actual} = $adrPkg->new( {
+		poBox           => $parts[0],
+		extendedAddress => $parts[1],
+		address         => $parts[2],
+		city            => $parts[3],
+		region          => $parts[4],
+		postalCode      => $parts[5],
+		country         => $parts[6],
+	});
 
     $_[0]->{$_[1]}{_pref}=$actual if $pref;
     delete $_[0]->{$_[1]}{_alias}{$actual};
